@@ -153,6 +153,9 @@ pub trait CSharpWriter {
             params.push(format!("{} {}", native, name));
         }
 
+
+        let return_stmt = if function.signature().rval().is_void() { "" } else { "return " };
+
         indented!(w, r#"public static {} {}({}) {{"#, rval, name, params.join(", "))?;
 
         for (pin_var, slice_struct) in to_pin_name.iter().zip(to_pin_slice_type.iter()) {
@@ -162,7 +165,7 @@ pub trait CSharpWriter {
 
         indented!(w, [_], r#"try"#)?;
         indented!(w, [_], r#"{{"#)?;
-        indented!(w, [_ _], r#"return {}({});"#, name, to_invoke.join(", "))?;
+        indented!(w, [_ _], r#"{}{}({});"#, return_stmt, name, to_invoke.join(", "))?;
         indented!(w, [_], r#"}}"#)?;
         indented!(w, [_], r#"finally"#)?;
         indented!(w, [_], r#"{{"#)?;
@@ -710,11 +713,29 @@ pub trait CSharpWriter {
         }
 
         let arg_tokens = names.iter().zip(types.iter()).map(|(n, t)| format!("{} {}", t, n)).collect::<Vec<_>>();
+        let fn_call = format!(r#"{}.{}(_context, {})"#, self.config().class, function.name(), names.join(", "));
 
-        indented!(w, r#"public {} {}({})"#, rval, fn_name, arg_tokens.join(","))?;
+        indented!(w, r#"public {} {}({})"#, rval, fn_name, arg_tokens.join(", "))?;
         indented!(w, r#"{{"#)?;
-        indented!(w, [_], r#"return {}.{}(_context, {});"#, self.config().class, function.name(), names.join(","))?;
+
+        match function.signature().rval() {
+            CType::Pattern(TypePattern::SuccessEnum(e)) => {
+                indented!(w, [_], r#"var rval = {};"#, fn_call)?;
+                indented!(w, [_], r#"if (rval != {}.{})"#, e.the_enum().rust_name(), e.success_variant().name())?;
+                indented!(w, [_], r#"{{"#)?;
+                indented!(w, [_ _], r#"throw new Exception("Something went wrong");"#)?;
+                indented!(w, [_], r#"}}"#)?;
+            }
+            CType::Primitive(PrimitiveType::Void) => {
+                indented!(w, [_], r#"{};"#, fn_call)?;
+            }
+            _ => {
+                indented!(w, [_], r#"return {};"#, fn_call)?;
+            }
+        }
+
         indented!(w, r#"}}"#)?;
+
         Ok(())
     }
 
